package com.example.data.repository

import com.example.data.database.DatabaseSettings
import com.example.data.database.tables.CartTable
import com.example.data.database.tables.FurnitureCategory
import com.example.data.database.tables.FurnitureTable
import com.example.data.database.toCartResponse
import com.example.dto.request.AddToCartRequest
import com.example.dto.request.ChangeCountFromCartRequest
import com.example.dto.request.RemoveFromCartRequest
import com.example.dto.response.CartResponse
import org.jetbrains.exposed.v1.core.SqlExpressionBuilder.eq
import org.jetbrains.exposed.v1.core.and
import org.jetbrains.exposed.v1.jdbc.deleteWhere
import org.jetbrains.exposed.v1.jdbc.insert
import org.jetbrains.exposed.v1.jdbc.selectAll
import org.jetbrains.exposed.v1.jdbc.update
import java.util.*

class CartRepository {
    suspend fun getCartByUuid(userUuid: UUID): List<CartResponse> = DatabaseSettings.dbQuery {
        val furniture  = (CartTable innerJoin FurnitureTable innerJoin FurnitureCategory)
            .selectAll()
            .where {
                CartTable.userUuid eq userUuid
            }
            .map {
                it.toCartResponse()
            }
        return@dbQuery furniture
    }
    suspend fun addToCartByUuid(addToCartRequest: AddToCartRequest) = DatabaseSettings.dbQuery {
        val result = CartTable.selectAll().where { CartTable.userUuid eq addToCartRequest.uuid and (CartTable.furnitureId eq addToCartRequest.furnitureId)}.firstOrNull()
        result?.let {
            CartTable.update(where = { CartTable.userUuid eq addToCartRequest.uuid and (CartTable.furnitureId eq addToCartRequest.furnitureId) }){
                it[count] = result[count] + 1
            }
            return@dbQuery true
        }
        return@dbQuery CartTable.insert {
            it[userUuid] = addToCartRequest.uuid
            it[furnitureId] = addToCartRequest.furnitureId
            it[count] = addToCartRequest.count
        }.insertedCount > 0
    }
    suspend fun removeFromCartByUuid(removeFromCartRequest: RemoveFromCartRequest) = DatabaseSettings.dbQuery {
        return@dbQuery CartTable.deleteWhere {
            userUuid eq removeFromCartRequest.uuid and (furnitureId eq removeFromCartRequest.furnitureId)
        } > 0
    }
    suspend fun changeCountFromCartByUuid(changeCountFromCartRequest: ChangeCountFromCartRequest) = DatabaseSettings.dbQuery {
        if (changeCountFromCartRequest.count == 0){
            return@dbQuery CartTable.deleteWhere {
                userUuid eq changeCountFromCartRequest.uuid and (furnitureId eq changeCountFromCartRequest.furnitureId)
            } > 0
        }
        return@dbQuery CartTable.update({ CartTable.userUuid eq changeCountFromCartRequest.uuid and (CartTable.furnitureId eq changeCountFromCartRequest.furnitureId) }) {
            it[count] = changeCountFromCartRequest.count
        } > 0
    }
}