package com.example.data.repository

import com.example.data.database.DatabaseSettings
import com.example.data.database.tables.FurnitureCategory
import com.example.data.database.tables.FurnitureTable
import com.example.data.database.tables.ShopCategoryFurnitureTable
import com.example.data.database.tables.ShopCategoryTable
import com.example.data.database.toFurniture
import com.example.data.database.toShopCategory
import com.example.dto.response.FurnitureResponse
import org.jetbrains.exposed.v1.core.SqlExpressionBuilder.eq
import org.jetbrains.exposed.v1.core.and
import org.jetbrains.exposed.v1.jdbc.andWhere
import org.jetbrains.exposed.v1.jdbc.selectAll

class FurnitureRepository {
    suspend fun getAllFurniture(): List<FurnitureResponse> = DatabaseSettings.dbQuery {
        val furniture = (FurnitureTable innerJoin FurnitureCategory).selectAll().map { it.toFurniture() }
            .map { furniture ->
            furniture.shopCategories = (FurnitureTable innerJoin ShopCategoryFurnitureTable innerJoin ShopCategoryTable)
                .selectAll()
                .where {
            FurnitureTable.id eq furniture.id }
                .map {
                    it.toShopCategory() }
                furniture
            }
        return@dbQuery furniture
    }
    suspend fun getFurnitureByCategory(categoryId: Long): List<FurnitureResponse> = DatabaseSettings.dbQuery {
        val furniture = (FurnitureTable innerJoin FurnitureCategory)
            .selectAll()
            .where { FurnitureTable.id eq categoryId }
            .map { it.toFurniture() }
            .map { furniture ->
            furniture.shopCategories = (FurnitureTable innerJoin ShopCategoryFurnitureTable innerJoin ShopCategoryTable)
                .selectAll()
                .where {
                    FurnitureTable.id eq furniture.id
                }.map {
                    it.toShopCategory() }
                furniture
            }
        return@dbQuery furniture
    }

    suspend fun getFurnitureById(furnitureId: Long): FurnitureResponse = DatabaseSettings.dbQuery {
        val furniture = (FurnitureTable innerJoin  FurnitureCategory)
            .selectAll()
            .where { FurnitureTable.id eq furnitureId }
            .firstOrNull()
        if(furniture == null) throw NoSuchElementException()
        return@dbQuery furniture.toFurniture()
    }
}