package com.example.data.repository

import com.example.data.database.DatabaseSettings
import com.example.data.database.tables.*
import com.example.data.database.tables.OrderTable.orderStatus
import com.example.data.database.toOrderItemResponse
import com.example.data.database.toOrderResponse
import com.example.dto.request.CreateOrderRequest
import com.example.dto.response.OrderResponse
import org.jetbrains.exposed.v1.core.SqlExpressionBuilder.eq
import org.jetbrains.exposed.v1.jdbc.*
import java.util.UUID

class OrderRepository {
   suspend fun createOrder(request: CreateOrderRequest): Boolean = DatabaseSettings.dbQuery {
       val order = OrderTable.insertReturning {
           it[orderTotalSum] = request.orderTotalSum
           it[orderStatus] = request.orderStatus
           it[userAddress]= request.addressId
           it[userUuid] = request.userUuid
       }.firstOrNull()
       order?.let {
           request.orderSet.forEach { orderSet ->
               OrderSetTable.insert {
                   it[orderId] = order[OrderTable.orderId]
                   it[furnitureId] = orderSet.furnitureId
                   it[furniturePrice] = orderSet.furniturePrice
                   it[count] = orderSet.count
               }
           }
           CartTable.deleteWhere { userUuid eq request.userUuid }
           return@dbQuery true
       }
       return@dbQuery false
   }
    suspend fun getOrdersByUuid(userUUID: UUID): List<OrderResponse> = DatabaseSettings.dbQuery {
        return@dbQuery (OrderTable innerJoin OrderStatusTable)
            .selectAll()
            .where {
                OrderTable.userUuid eq userUUID
            }
            .map { it.toOrderResponse() }
            .map {  order ->
                order.orderSet = (OrderSetTable innerJoin FurnitureTable innerJoin FurnitureCategory).selectAll()
                    .where { OrderSetTable.orderId eq order.id }
                    .map {
                        it.toOrderItemResponse()
                    }
                order
            }
            .sortedBy { it.dateTime }
    }
}