package com.example.data.repository

import com.example.data.database.DatabaseSettings
import com.example.data.database.tables.FurnitureCategory
import com.example.data.database.tables.FurnitureTable
import com.example.data.database.tables.ShopCategoryFurnitureTable
import com.example.data.database.tables.ShopCategoryTable
import com.example.data.database.toAllShopCategoryResponse
import com.example.data.database.toFurniture
import com.example.data.database.toShopCategory
import com.example.dto.response.AllShopCategoryResponse
import com.example.dto.response.FurnitureResponse
import org.jetbrains.exposed.v1.jdbc.selectAll

class ShopCategoryRepository {
    suspend fun getAllShopCategory(): List<AllShopCategoryResponse> = DatabaseSettings.dbQuery {
        val result = ShopCategoryTable
            .selectAll()
            .map {
                it.toAllShopCategoryResponse()
            }.map { category ->
                category.furnitureList = (FurnitureTable innerJoin ShopCategoryFurnitureTable innerJoin FurnitureCategory)
                    .selectAll()
                    .where { ShopCategoryFurnitureTable.shopCategoryId eq category.id}
                    .map {
                        it.toFurniture()
                    }
                category
            }
        return@dbQuery result
    }
    suspend fun getFurnitureByShopCategory(shopCategoryId: Long): List<FurnitureResponse> = DatabaseSettings.dbQuery {
        val furniture = (ShopCategoryFurnitureTable innerJoin FurnitureTable innerJoin ShopCategoryTable innerJoin FurnitureCategory).selectAll()
            .where { ShopCategoryFurnitureTable.shopCategoryId eq shopCategoryId }
            .map { it.toFurniture() }

        return@dbQuery furniture
    }
}