package com.example.data.repository

import com.example.data.database.DatabaseSettings
import com.example.data.database.tables.AddressTable
import com.example.data.database.tables.ProfileTable
import com.example.data.database.tables.UserTable
import com.example.dto.request.AddAddressRequest
import com.example.dto.request.LoginRequest
import com.example.dto.request.RegisterRequest
import com.example.dto.response.AddressResponse
import com.example.dto.response.UserResponse
import org.jetbrains.exposed.v1.core.SqlExpressionBuilder.eq
import org.jetbrains.exposed.v1.jdbc.*
import java.util.UUID

class UserRepository {
    suspend fun register(registerRequest: RegisterRequest): UserResponse{
       return DatabaseSettings.dbQuery {
            val user = UserTable.insertReturning {
                it[email] = registerRequest.email
                it[password] = registerRequest.password
            }.firstOrNull()

           if(user == null) throw NullPointerException("User not created")
            val result = ProfileTable.insertReturning {
                it[uuid] = user[UserTable.uuid]
                it[firstName] = registerRequest.firstName
                it[lastName] = registerRequest.lastName
            }.firstOrNull()
           if(result == null) throw NullPointerException("Profile not created")
           val userResponse = UserResponse(
               userUuid = user[UserTable.uuid],
               firstName = result[ProfileTable.firstName],
               lastName = result[ProfileTable.lastName],
               email = user[UserTable.email],
               imageUrl = result[ProfileTable.imageUrl]
           )
           return@dbQuery userResponse
        }
    }

    suspend fun login(loginRequest: LoginRequest): UserResponse {
       return DatabaseSettings.dbQuery {
            val user = (UserTable innerJoin ProfileTable).selectAll().where { UserTable.email eq loginRequest.email }.firstOrNull()
            user?.let {
                if(it[UserTable.password] != loginRequest.password) throw IllegalArgumentException("Uncorrected password")
                return@dbQuery UserResponse(
                    userUuid = it[ProfileTable.uuid],
                    email = it[UserTable.email],
                    firstName = it[ProfileTable.firstName],
                    lastName = it[ProfileTable.lastName],
                    imageUrl = it[ProfileTable.imageUrl]
                )
            }
            throw NullPointerException("User not found")
        }
    }

    suspend fun getUserByUuid(uuid: UUID): UserResponse = DatabaseSettings.dbQuery{
        val user = (UserTable innerJoin  ProfileTable leftJoin  AddressTable).selectAll().where { UserTable.uuid eq uuid }.firstOrNull()
        if(user == null) throw NullPointerException("User not found")
        val userResponse = UserResponse(
            userUuid = user[ProfileTable.uuid],
            email = user[UserTable.email],
            firstName = user[ProfileTable.firstName],
            lastName = user[ProfileTable.lastName],
            imageUrl = user[ProfileTable.imageUrl],
        )
        if (user.hasValue(AddressTable.id)){
            userResponse.address = AddressResponse(
                addressId = user[AddressTable.id],
                entrance = user[AddressTable.entrance],
                apartment = user[AddressTable.apartment],
                floor = user[AddressTable.floor],
                comment = user[AddressTable.comment],
                address = user[AddressTable.address],
            )
        }
        return@dbQuery userResponse
    }

    suspend fun addAddressByUuid(uuid: UUID, addAddressRequest: AddAddressRequest) = DatabaseSettings.dbQuery {
            val user = (ProfileTable leftJoin  AddressTable).selectAll().where { ProfileTable.uuid eq uuid }.firstOrNull()
            if(user == null) throw NullPointerException("User not found")
            val address = AddressTable.insertReturning {
                it[comment] = addAddressRequest.comment
                it[address] = addAddressRequest.address
                it[apartment] = addAddressRequest.apartment
                it[floor] = addAddressRequest.floor
                it[entrance] = addAddressRequest.entrance
            }.firstOrNull()
        if(address == null) throw NullPointerException("Address not createad")
        ProfileTable.update({ ProfileTable.uuid eq uuid }) {
                it[ProfileTable.address] = address[AddressTable.id]
            }
        if(user.hasValue(AddressTable.id)){
            AddressTable.deleteWhere { id eq user[id] }
        }
    }
}