package com.example.data.repository

import com.example.data.database.DatabaseSettings
import com.example.data.database.tables.FurnitureCategory
import com.example.data.database.tables.FurnitureTable
import com.example.data.database.tables.WishlistTable
import com.example.data.database.toFurniture
import com.example.dto.request.AddToWishlistRequest
import com.example.dto.request.RemoveFromWishlistRequest
import com.example.dto.response.FurnitureResponse
import org.jetbrains.exposed.v1.core.SqlExpressionBuilder.eq
import org.jetbrains.exposed.v1.core.and
import org.jetbrains.exposed.v1.jdbc.deleteReturning
import org.jetbrains.exposed.v1.jdbc.deleteWhere
import org.jetbrains.exposed.v1.jdbc.insert
import org.jetbrains.exposed.v1.jdbc.selectAll
import java.util.*

class WishlistRepository {
    suspend fun getWishListByUuid(userUuid: UUID): List<FurnitureResponse> = DatabaseSettings.dbQuery {
        val furniture  = (WishlistTable innerJoin FurnitureTable innerJoin FurnitureCategory)
            .selectAll()
            .where {
                WishlistTable.userUuid eq userUuid
            }
            .map {
                it.toFurniture()
            }
        return@dbQuery furniture
    }
    suspend fun addToWishListByUuid(addToWishlistRequest: AddToWishlistRequest): Boolean = DatabaseSettings.dbQuery {
        val result = WishlistTable.insert {
            it[userUuid] = addToWishlistRequest.uuid
            it[furnitureId] = addToWishlistRequest.furnitureId
        }
        return@dbQuery result.insertedCount > 0
    }

    suspend fun removeFromWishlistByUuid(removeFromWishlistRequest: RemoveFromWishlistRequest): Boolean = DatabaseSettings.dbQuery {
        return@dbQuery WishlistTable.deleteWhere {
            userUuid eq removeFromWishlistRequest.uuid and (furnitureId eq removeFromWishlistRequest.furnitureId)
        } > 0
    }
}