package com.example.route

import com.example.data.repository.CartRepository
import com.example.data.repository.OrderRepository
import com.example.data.repository.UserRepository
import com.example.data.repository.WishlistRepository
import com.example.dto.request.AddAddressRequest
import com.example.dto.request.AddToWishlistRequest
import com.example.dto.request.LoginRequest
import com.example.dto.request.RegisterRequest
import com.example.dto.response.AddressResponse
import io.ktor.http.*
import io.ktor.server.request.*
import io.ktor.server.response.*
import io.ktor.server.routing.*
import java.util.UUID

fun Route.userRoute() {
    val userRepository = UserRepository()
    route("/auth") {
        post("/login") {
            val request = call.receive<LoginRequest>()
            try {
                val result = userRepository.login(request)
                call.respond(HttpStatusCode.OK, result)
            }
            catch (e: Exception) {
                call.respond(HttpStatusCode.NotFound)
            }
        }
        post("/register") {
            try {
                val request = call.receive<RegisterRequest>()
                val result = userRepository.register(request)
                call.respond(HttpStatusCode.OK, result)
            } catch (e: Exception) {
                call.respond(HttpStatusCode.NotFound)
            }
        }
    }
    route("/user"){
        val wishlistRepository = WishlistRepository()
        val cartRepository = CartRepository()
        val orderRepository = OrderRepository()
        get("{uuid}/wishlist"){
            val uuid = call.parameters["uuid"] ?: return@get call.respond(HttpStatusCode.BadRequest)
            val getByUuidCommand = UUID.fromString(uuid)
            val result = wishlistRepository.getWishListByUuid(getByUuidCommand)
            call.respond(HttpStatusCode.OK, result)
        }
        get("{uuid}/cart"){
            val uuid = call.parameters["uuid"] ?: return@get call.respond(HttpStatusCode.BadRequest)
            val getByCartCommand = UUID.fromString(uuid)
            val result = cartRepository.getCartByUuid(getByCartCommand)
            call.respond(HttpStatusCode.OK, result)
        }
        get("{uuid}/order") {
            val uuid = call.parameters["uuid"] ?: return@get call.respond(HttpStatusCode.BadRequest)
            val getByOrderCommand = UUID.fromString(uuid)
            val result = orderRepository.getOrdersByUuid(getByOrderCommand)
            call.respond(HttpStatusCode.OK, result)
        }
        get("{uuid}/profile") {
            val uuid = call.parameters["uuid"] ?: return@get call.respond(HttpStatusCode.BadRequest)
            val getByOrderCommand = UUID.fromString(uuid)
            val result = userRepository.getUserByUuid(getByOrderCommand)
            call.respond(HttpStatusCode.OK, result)
        }
        post("{uuid}/address") {
            val uuid = call.parameters["uuid"] ?: return@post call.respond(HttpStatusCode.BadRequest)
            val getByOrderCommand = UUID.fromString(uuid)
            val address = call.receive<AddAddressRequest>()
            val result = userRepository.addAddressByUuid(getByOrderCommand, address)
            call.respond(HttpStatusCode.OK, result)
        }
    }
}